/*************************************
**function : ADC NO_DMA
** PORTC.0 -> analog input(ADC CH10)
**date     : 2022/03/1
**programmed by ksc
***************************************/
void ADC_Calib(void);
void ADC_Init(void);
void SEND_ADC(float);
void Delay(unsigned long);
/*************************************************************
**define reset and clock control Register RCC address
**RCC_base_address 0x40021000
**************************************************************/
#define RCC_CR_address        0x40021000  //clock control register
#define RCC_CFGR_address      0x40021004  //clock configuration register
#define RCC_CIR_address       0x40021008  //clock interrupt register
#define RCC_APB2RSTR_address  0x4002100C  //APB2 peripheral reset register
#define RCC_APB1RSTR_address  0x40021010  //APB1 peripheral reset register 
#define RCC_AHBENR_address    0x40021014  // AHB peripheral clock enable register
#define RCC_APB2ENR_address   0x40021018  // APB2 peripheral clock enable register
#define RCC_APB1ENR_address   0x4002101C  // APB2 peripheral clock enable register
#define RCC_BDCR_address      0x40021020  // backup domain control register
#define RCC_CSR_address       0x40021020  // control/status register
/************************************************************
//Flash Memory Control Register address
************************************************************/
#define Flash_ACR_address 0x40022000

/******************************************
**define GPIOA register address
**portA_base_address  0x40010c00
*******************************************/
#define GPIOA_CRL_address    0x40010800
#define GPIOA_CRH_address    0x40010804
#define GPIOA_IDR_address    0x40010808
#define GPIOA_ODR_address    0x4001080c
#define GPIOA_BSRR_address   0x40010810
#define GPIOA_BRR_address    0x40010814
#define GPIOA_CLKR_address   0x40010818

/*******************************************
**define GPIOB register address
**portB_base_address  0x40010c00
*******************************************/
#define GPIOB_CRL_address    0x40010c00
#define GPIOB_CRH_address    0x40010c04
#define GPIOB_IDR_address    0x40010c08
#define GPIOB_ODR_address    0x40010c0c
#define GPIOB_BSRR_address   0x40010c10
#define GPIOB_BRR_address    0x40010c14
#define GPIOB_CLKR_address   0x40010c18
/*******************************************
**define GPIOC register address
**portC_base_address  0x40011000
*******************************************/
#define GPIOC_CRL_address    0x40011000
#define GPIOC_CRH_address    0x40011004
#define GPIOC_IDR_address    0x40011008
#define GPIOC_ODR_address    0x4001100c
#define GPIOC_BSRR_address   0x40011010
#define GPIOC_BRR_address    0x40011014
#define GPIOC_CLKR_address   0x40011018
/******************************************************
** define USART register address
** USART base address : 0x4001 3800 - 0x4001 3BFF
*****************************************************/
#define USART_SR_address      0x40013800
#define USART_DR_address      0x40013804
#define USART_BRR_address     0x40013808
#define USART_CR1_address     0x4001380C
#define USART_CR2_address     0x40013810
#define USART_CR3_address     0x40013814
/***************************************************
**define ADC1 register address
**ADC1_base_address  0x4001 2400 - 0x4001 27FF
***************************************************/
#define ADC1_SR_address          0x40012400
#define ADC1_CR1_address         0x40012404
#define ADC1_CR2_address         0x40012408
#define ADC1_SMPR1_address       0x4001240C
#define ADC1_SMPR2_address       0x40012410
#define ADC1_JOFR1_address       0x40012414
#define ADC1_JOFR2_address       0x40012418
#define ADC1_JOFR3_address       0x4001241C
#define ADC1_JOFR4_address       0x40012420
#define ADC1_HTR_address         0x40012424
#define ADC1_LTR_address         0x40012428
#define ADC1_SQR1_address        0x4001242C
#define ADC1_SQR2_address        0x40012430
#define ADC1_SQR3_address        0x40012434
#define ADC1_JSQR_address        0x40012438
#define ADC1_JDR1_address        0x4001243C
#define ADC1_JDR2_address        0x40012440
#define ADC1_JDR3_address        0x40012444
#define ADC1_JDR4_address        0x40012448
#define ADC1_DR_address          0x4001244C

void main()
{
 //unsigned int ADC_DATA_LOW, ADC_DATA_HIGH;
 unsigned long ADC_DATA;
 float volt;
/*************************************************************************/
// Initialize CLOCK Parameter
/*************************************************************************/  
(*(volatile unsigned *)RCC_CR_address)|=0x00010000;       //HSE ON enable
(*(volatile unsigned *)Flash_ACR_address)|=0x00000002;    //set flash access latency
(*(volatile unsigned *)RCC_CFGR_address)|=0x00000400;     //PPRE2=0,PPRE1=/2,HPERE=0
(*(volatile unsigned *)RCC_CFGR_address)|=0x00110000;     //PLL SOURCE =HSE, PLL XTPRE=0, PLLMUL= x6
(*(volatile unsigned *)RCC_CFGR_address)|=0x00008000;     //ADCPRE=APB2/6=12MHz
(*(volatile unsigned *)RCC_AHBENR_address)|=0x0000000;    // DMA Disable
(*(volatile unsigned *)RCC_CR_address)|=0x01010000;       //PLL enable
(*(volatile unsigned *)RCC_CIR_address)|=0x00000000;      // all clock interrupt disable
(*(volatile unsigned *)RCC_CFGR_address)|=0x00000002;     //PLL selected as system clock
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x400d;      //PortA,B USART clock enable   
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x0000001C;  //PortB&PortA&PORTC clock enable  

/**************************************************************************/
//PORTA : 9,10 bit 
//Bit9 :output mode , push-pull,50MHz
//bit10 : input mode floating
/**************************************************************************/
(*(volatile unsigned *)GPIOA_CRH_address)=0x4b0;   //1011 0000,bit9,output,pushpull,50MHz
//CNF : general purpose open drain output
//Mode : output mode(max speed : 50MHz)
(*(volatile unsigned *)GPIOB_CRL_address)=0x00700000; //set bit5 mode&cnf
//CNF : general purpose open drain output
//Mode : output mode(max speed : 50MHz)
(*(volatile unsigned *)GPIOB_CRH_address)=0x00000077; //set bit8,9 mode&cnf

/***************************************************************************/
// USART Initialize
// Baud-rate :115200
// data length :8 bit
// 1 stop bit
// No parity
// No hardware control
// TX,TX enable
/**************************************************************************/
(*(volatile unsigned *)USART_BRR_address)=0x271 ;    //set baud rate 
(*(volatile unsigned *)USART_CR1_address)=0x2008;    //10 0000 0000 1100
(*(volatile unsigned *)USART_CR2_address)=0x00;
(*(volatile unsigned *)USART_CR3_address)=0x00;
(*(volatile unsigned *)USART_DR_address)=0x30;
/**************************************************************************/
//key : portA bit0,bit1
//CNF : Floating input input
//mode : input mode
/**************************************************************************/
(*(volatile unsigned *)GPIOA_CRL_address)|=0x00000044;
(*(volatile unsigned *)GPIOA_CRH_address)|=0x00000000;
/**************************************************************************/
//CNF : general purpose open drain output
//Mode : output mode(open drain, max speed : 50MHz)
(*(volatile unsigned *)GPIOB_CRL_address)=0x00700000; //set bit5 mode&cnf

/**************************************************************************/
//CNF : general purpose open drain output
//Mode : output mode(open drain, max speed : 50MHz)
/**************************************************************************/
(*(volatile unsigned *)GPIOB_CRH_address)=0x00000077; //set bit8,9 mode&cnf
(*(volatile unsigned *)GPIOB_ODR_address)=0x0320;

/***************************************************************************/
//ADC register set
/****************************************************************************/
ADC_Init();
ADC_Calib();
while(1){
(*(volatile unsigned *)ADC1_CR2_address)|=0x00400010;   // ַ ä ȯ 
while( !(((*(volatile unsigned *)ADC1_SR_address))&0x02)>>1);// wait untill EOC=1
ADC_DATA=(*(volatile unsigned *)ADC1_DR_address);  // Read ADC Data Register
volt=(float)ADC_DATA*3.3/(float)4095;

SEND_ADC(volt);

(*(volatile unsigned *)GPIOB_ODR_address)=0x0200; 
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
(*(volatile unsigned *)GPIOB_ODR_address)=0x0300;
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
Delay(1000);
}
}
void ADC_Calib(void){
 (*(volatile unsigned *)ADC1_CR2_address)|=0x04;  //start calibration CAL=1
while((*(volatile unsigned *)ADC1_CR2_address)&0x04);  
}
void ADC_Init(void){
(*(volatile unsigned *)RCC_CFGR_address)|=0x00008000;       //ADC prescaler=div6 , 12MHz  
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x00000200;    //ADC1 clock enable
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x00000001;    //Alternate function clock enable
//(*(volatile unsigned *)RCC_AHBENR_address)|=0x00000001;     //DMA1 clock Enable
// PORTC.0 -> analog input(ADC CH10)
(*(volatile unsigned *)GPIOC_CRL_address)|=0x00000000;      
(*(volatile unsigned *)GPIOC_CRH_address)|=0x00000000;

// ADC Start flag, EOC clear
(*(volatile unsigned *)ADC1_SR_address)|=0x00000000; 
//Discontinuous mode on regular channels, 1 channel,interrupt disabled
(*(volatile unsigned *)ADC1_CR1_address)|=0x00000800; 
//Start conversion of regular channels, DMA Disabled,A/D converter ON, Right Alignment
(*(volatile unsigned *)ADC1_CR2_address)|=0x00400001;  //0000 0000 0100 0000 0000 0000 0000 0001
//External trigger=1
(*(volatile unsigned *)ADC1_CR2_address)|=0x00100000;        
//ܺ Ʈ  : Ʈ Ʈ 
(*(volatile unsigned *)ADC1_CR2_address)|=0x000E0000;        //EXTSEL=111-> SWSTART
// ä10 sampling time set 55.5cycle
(*(volatile unsigned *)ADC1_SMPR1_address)|=0x00000005;      
//Regular channel sequence length=0000, 1 conversion
(*(volatile unsigned *)ADC1_SQR1_address)|=0x00000000;
(*(volatile unsigned *)ADC1_SQR2_address)|=0x00000000;
//Select 1st conversion in regular sequence= ch10
(*(volatile unsigned *)ADC1_SQR3_address)|=0x0000000A;
}
void Delay(unsigned long nTime) {
  unsigned long TimingDelay;
  TimingDelay=nTime;
  while(TimingDelay != 0){
    TimingDelay--;
}
}
void SEND_ADC(float volt_val){
unsigned int volt_decimal,volt_fraction;   
volt_decimal=(unsigned int)volt_val;   //  ϱ
volt_fraction=(unsigned int)((volt_val-volt_decimal)*10); // Ҽ ù°ڸ ϱ  
// 
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=(volt_decimal+0x30);
//DOT 
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=(0x2E);
//Ҽ 
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=(volt_fraction+0x30);
// 'V' 
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=(0x56);
//ٹٲ ĳ 
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=0x0d;
while( !(((*(volatile unsigned *)USART_SR_address)&0x80)>>7));      //wait untill TXE=1
(*(volatile unsigned *)USART_DR_address)=0x0A;  
  
  
  
  
  
  
}

